@@ -257,7 +257,7 @@ int capng_get_caps_process(void)
 }
 
 #ifdef VFS_CAP_U32
-static int load_data(struct vfs_cap_data *filedata, int size)
+static int load_data(const struct vfs_cap_data *filedata, int size)
 {
 	unsigned int magic;
 	
@@ -549,8 +549,9 @@ int capng_change_id(int uid, int gid, capng_flags_t flag)
 
 	// Check the current capabilities
 #ifdef PR_CAPBSET_DROP
-	// If newer kernel, we need setpcap
-	if (capng_have_capability(CAPNG_EFFECTIVE, CAP_SETPCAP) == 0)
+	// If newer kernel, we need setpcap to change the bounding set
+	if (capng_have_capability(CAPNG_EFFECTIVE, CAP_SETPCAP) == 0 && 
+					flag & CAPNG_CLEAR_BOUNDING)
 		capng_update(CAPNG_ADD,
 				CAPNG_EFFECTIVE|CAPNG_PERMITTED, CAP_SETPCAP);
 #endif
@@ -847,6 +848,8 @@ char *capng_print_caps_numeric(capng_print_t where, capng_select_t set)
 			char *s;
 			if (ptr == NULL ){
 				ptr = malloc(40);
+				if (ptr == NULL)
+					return ptr;
 				*ptr = 0;
 				s = ptr;
 			} else
